using System;
using System.Collections.Generic;
using System.Linq;

namespace Prova1.Model
{
    public class FillInTheBlanksQuestion : Question
    {
        private readonly IList<IPart> _parts;
        private readonly Dictionary<EditablePart, string> _givenAnswers;

        public FillInTheBlanksQuestion(IList<IPart> parts)
        {
            if (parts == null)
                throw new ArgumentNullException("parts");
            if (parts.Count == 0)
                throw new ArgumentException("parts.Count == 0");
            _parts = parts;
            _givenAnswers = new Dictionary<EditablePart, string>();
        }

        public IEnumerable<IPart> Parts
        {
            get { return _parts; }
        }

        public void SetGivenAnswer(EditablePart editablePart, string text)
        {
            if (!Parts.Contains(editablePart))
                throw new ArgumentException("!Parts.Contains(editablePart)");
            _givenAnswers[editablePart] = text;
        }

        public override double GetScore()
        {
            double score = 0;
            foreach (EditablePart editablePart in _parts.OfType<EditablePart>())
            {
                string answer;
                _givenAnswers.TryGetValue(editablePart, out answer);
                if (!String.IsNullOrEmpty(answer))
                {
                    if (String.Equals(editablePart.ExpectedText, answer, StringComparison.InvariantCultureIgnoreCase))
                    {
                        score += editablePart.RightAnswerPoints;
                    }
                    else
                    {
                        score += editablePart.WrongAnswerPoints;
                    }
                }
            }
            return score;
        }
    }
}