﻿using System;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Collections.Generic;

using Prova1.Model;
using Prova1.Persistence;

namespace Prova1.Presentation
{
    public class MainController
    {
        private readonly Exam _exam;
        private readonly IMainView _mainView;
        private readonly IQuestionControlFactory _questionControlFactory;

        public MainController(Exam exam, IMainView mainView, IQuestionControlFactory questionControlFactory)
        {
            if (exam == null)
                throw new ArgumentNullException("exam");
            if (mainView == null)
                throw new ArgumentNullException("mainView");
            if (questionControlFactory == null)
                throw new ArgumentNullException("questionControlFactory");
            _exam = exam;
            _mainView = mainView;
            _mainView.StartButtonEnabled = true;
            _mainView.NextButtonEnabled = false;
            _mainView.StartButtonClick += new EventHandler(_mainView_StartButtonClick);
            _mainView.NextButtonClick += new EventHandler(_mainView_NextButtonClick);
            _questionControlFactory = questionControlFactory;
        }

        private void _mainView_StartButtonClick(object sender, EventArgs e)
        {
            _mainView.StartButtonEnabled = false;
            _mainView.NextButtonEnabled = true;
            _exam.CurrentQuestionIndex = 0;
            UpdateUI();
        }

        private void _mainView_NextButtonClick(object sender, EventArgs e)
        {
            _mainView.QuestionControl.SetGivenAnswer();
            _exam.CurrentQuestionIndex++;
            UpdateUI();
        }

        private void UpdateUI()
        {
            _mainView.StatusText = String.Format("Current score: {0}", _exam.GetScore());
            if (_exam.CurrentQuestion != null)
            {
                _mainView.QuestionControl = _questionControlFactory.GetQuestionControlFor(_exam.CurrentQuestion);
            }
            else
            {
                _mainView.NextButtonEnabled = false;
                _mainView.QuestionControl = null;
            }
        }
    }
}
