﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Windows.Forms;

using Prova1.Model;

namespace Prova1.Presentation
{
    public class QuestionControlFactory : IQuestionControlFactory
    {
        private readonly Dictionary<Type, Type> _questionControlDictionary = new Dictionary<Type, Type>();

        public QuestionControlFactory(params Assembly[] assemblies)
        {
            if (assemblies == null)
                throw new ArgumentNullException("assemblies");
            foreach (Assembly assembly in assemblies)
            {
                foreach (Type type in assembly.GetTypes())
                {
                    if (typeof(IQuestionControl).IsAssignableFrom(type) && !type.IsAbstract)
                    {
                        QuestionTypeAttribute[] attributes = (QuestionTypeAttribute[]) type.GetCustomAttributes(typeof(QuestionTypeAttribute), false);
                        if (attributes.Length > 0)
                        {
                            _questionControlDictionary.Add(attributes[0].QuestionType, type);
                        }
                    }
                }
            }
        }

        public IQuestionControl GetQuestionControlFor(Question question)
        {
            Type questionType = question.GetType();
            if (!_questionControlDictionary.ContainsKey(questionType))
                throw new ApplicationException("No control for question: " + questionType.Name);
            Type controlType = _questionControlDictionary[questionType];
            return (IQuestionControl) Activator.CreateInstance(controlType, new object[] { question });
        }
    }
}
