﻿using System;
using System.Reflection;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;

using Prova1.Model;
using Prova1.Presentation;

namespace Prova1Test
{


    /// <summary>
    ///This is a test class for MainControllerTest and is intended
    ///to contain all MainControllerTest Unit Tests
    ///</summary>
    [TestClass()]
    public class MainControllerTest
    {


        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        // 
        //You can use the following additional attributes as you write your tests:
        //
        //Use ClassInitialize to run code before running the first test in the class
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Use ClassCleanup to run code after all tests in a class have run
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Use TestInitialize to run code before running each test
        //[TestInitialize()]
        //public void MyTestInitialize()
        //{
        //}
        //
        //Use TestCleanup to run code after each test has run
        //[TestCleanup()]
        //public void MyTestCleanup()
        //{
        //}
        //
        #endregion


        /// <summary>
        ///A test for MainController Constructor
        ///</summary>
        [TestMethod()]
        public void MainControllerConstructorTest()
        {
            List<Question> questions = new List<Question> { new DummyQuestion(10), new DummyQuestion(20) };
            Exam exam = new Exam(questions);
            DummyMainView mainView = new DummyMainView();
            IQuestionControlFactory questionControlFactory = new QuestionControlFactory(Assembly.GetExecutingAssembly());
            MainController target = new MainController(exam, mainView, questionControlFactory);
            Assert.IsTrue(mainView.StartButtonEnabled);
            Assert.IsFalse(mainView.NextButtonEnabled);
            mainView.ClickStartButton();
            Assert.IsFalse(mainView.StartButtonEnabled);
            Assert.IsTrue(mainView.NextButtonEnabled);
            Assert.AreEqual(0, exam.CurrentQuestionIndex);
            Assert.AreEqual(questions[0], exam.CurrentQuestion);
            Assert.IsTrue(mainView.StatusText.Contains(exam.GetScore().ToString()));
            Assert.IsInstanceOfType(mainView.QuestionControl, typeof(DummyQuestionControl));
            mainView.ClickNextButton();
            Assert.IsFalse(mainView.StartButtonEnabled);
            Assert.IsTrue(mainView.NextButtonEnabled);
            Assert.AreEqual(1, exam.CurrentQuestionIndex);
            Assert.AreEqual(questions[1], exam.CurrentQuestion);
            Assert.IsTrue(mainView.StatusText.Contains(exam.GetScore().ToString()));
            Assert.IsInstanceOfType(mainView.QuestionControl, typeof(DummyQuestionControl));
            mainView.ClickNextButton();
            Assert.IsFalse(mainView.StartButtonEnabled);
            Assert.IsFalse(mainView.NextButtonEnabled);
            Assert.AreEqual(2, exam.CurrentQuestionIndex);
            Assert.IsNull(exam.CurrentQuestion);
            Assert.IsTrue(mainView.StatusText.Contains(exam.GetScore().ToString()));
            Assert.IsNull(mainView.QuestionControl);
        }

        private class DummyMainView : IMainView
        {
            public void ClickStartButton()
            {
                if (StartButtonClick != null)
                    StartButtonClick(this, EventArgs.Empty);
            }

            public void ClickNextButton()
            {
                if (NextButtonClick != null)
                    NextButtonClick(this, EventArgs.Empty);
            }

            #region IMainView Members

            public event EventHandler StartButtonClick;

            public event EventHandler NextButtonClick;

            public bool StartButtonEnabled
            {
                get;
                set;
            }

            public bool NextButtonEnabled
            {
                get;
                set;
            }

            public IQuestionControl QuestionControl
            {
                get;
                set;
            }

            public string StatusText
            {
                get;
                set;
            }

            #endregion
        }
    }
}
