﻿using Prova1.Persistence;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using System.Xml;
using System.Linq;
using Prova1.Model;
using System.Collections.Generic;

namespace Prova1Test
{


    /// <summary>
    ///This is a test class for XmlExamReaderTest and is intended
    ///to contain all XmlExamReaderTest Unit Tests
    ///</summary>
    [TestClass()]
    public class XmlExamReaderTest
    {


        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        // 
        //You can use the following additional attributes as you write your tests:
        //
        //Use ClassInitialize to run code before running the first test in the class
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Use ClassCleanup to run code after all tests in a class have run
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Use TestInitialize to run code before running each test
        //[TestInitialize()]
        //public void MyTestInitialize()
        //{
        //}
        //
        //Use TestCleanup to run code after each test has run
        //[TestCleanup()]
        //public void MyTestCleanup()
        //{
        //}
        //
        #endregion

        private string _multipleChoiceText = @"<?xml version=""1.0"" encoding=""utf-8"" ?>
<exam>
  <multipleChoice id=""Q1"">
    <text>Di che colore era il cavallo bianco di Napoleone?</text>
    <answer points=""1"">Grigio</answer>
    <answer points=""-0.25"">Verde</answer>
    <answer points=""-0.25"">Blu</answer>
    <answer points=""-0.25"">Nero</answer>
  </multipleChoice>
</exam>";

        private static void CheckMultipleChoiceQuestion(Question actual)
        {
            Assert.IsInstanceOfType(actual, typeof(MultipleChoiceQuestion));
            MultipleChoiceQuestion question = (MultipleChoiceQuestion) actual;
            Assert.AreEqual("Q1", question.Id);
            Assert.AreEqual("Di che colore era il cavallo bianco di Napoleone?", question.Text);
            Assert.AreEqual(4, question.Answers.Count());
            Answer answer = question.Answers.ElementAt(0);
            Assert.AreEqual(1, answer.Points);
            Assert.AreEqual("Grigio", answer.Text);
            answer = question.Answers.ElementAt(1);
            Assert.AreEqual(-0.25, answer.Points);
            Assert.AreEqual("Verde", answer.Text);
            answer = question.Answers.ElementAt(2);
            Assert.AreEqual(-0.25, answer.Points);
            Assert.AreEqual("Blu", answer.Text);
            answer = question.Answers.ElementAt(3);
            Assert.AreEqual(-0.25, answer.Points);
            Assert.AreEqual("Nero", answer.Text);
        }

        private string _fillInTheBlanksText = @"<?xml version=""1.0"" encoding=""utf-8"" ?>
<exam>
  <fillInTheBlanks id=""Q3"">
    <text>Completa la seguente frase.</text>
    <toComplete>
      <text>Nel mezzo del cammin di nostra</text>
      <blank right=""1"" wrong=""-0.25"">vita</blank>
      <text>mi ritrovai per una</text>
      <blank right=""1"" wrong=""-0.25"">selva</blank>
      <text>oscura...</text>
    </toComplete>
  </fillInTheBlanks>
</exam>";

        private static void CheckFillInTheBlanksQuestion(Question actual)
        {
            Assert.IsInstanceOfType(actual, typeof(FillInTheBlanksQuestion));
            FillInTheBlanksQuestion question = (FillInTheBlanksQuestion) actual;
            Assert.AreEqual("Q3", question.Id);
            Assert.AreEqual("Completa la seguente frase.", question.Text);
            Assert.AreEqual(5, question.Parts.Count());
            Assert.IsInstanceOfType(question.Parts.ElementAt(0), typeof(TextPart));
            TextPart textPart = (TextPart) question.Parts.ElementAt(0);
            Assert.AreEqual("Nel mezzo del cammin di nostra", textPart.Text);
            Assert.IsInstanceOfType(question.Parts.ElementAt(1), typeof(EditablePart));
            EditablePart editablePart = (EditablePart) question.Parts.ElementAt(1);
            Assert.AreEqual("vita", editablePart.ExpectedText);
            Assert.AreEqual(1, editablePart.RightAnswerPoints);
            Assert.AreEqual(-0.25, editablePart.WrongAnswerPoints);
            Assert.IsInstanceOfType(question.Parts.ElementAt(2), typeof(TextPart));
            textPart = (TextPart) question.Parts.ElementAt(2);
            Assert.AreEqual("mi ritrovai per una", textPart.Text);
            Assert.IsInstanceOfType(question.Parts.ElementAt(3), typeof(EditablePart));
            editablePart = (EditablePart) question.Parts.ElementAt(3);
            Assert.AreEqual("selva", editablePart.ExpectedText);
            Assert.AreEqual(1, editablePart.RightAnswerPoints);
            Assert.AreEqual(-0.25, editablePart.WrongAnswerPoints);
            Assert.IsInstanceOfType(question.Parts.ElementAt(4), typeof(TextPart));
            textPart = (TextPart) question.Parts.ElementAt(4);
            Assert.AreEqual("oscura...", textPart.Text);
        }

        /// <summary>
        ///A test for ReadQuestions
        ///</summary>
        [TestMethod()]
        public void ReadQuestionsTest1()
        {
            TextReader reader = new StringReader(_multipleChoiceText);
            XmlExamReader target = new XmlExamReader(reader);
            IList<Question> questions = target.ReadQuestions();
            Assert.AreEqual(1, questions.Count);
            CheckMultipleChoiceQuestion(questions[0]);
        }

        /// <summary>
        ///A test for ReadQuestions
        ///</summary>
        [TestMethod()]
        public void ReadQuestionsTest2()
        {
            TextReader reader = new StringReader(_fillInTheBlanksText);
            XmlExamReader target = new XmlExamReader(reader);
            IList<Question> questions = target.ReadQuestions();
            Assert.AreEqual(1, questions.Count);
            CheckFillInTheBlanksQuestion(questions[0]);
        }
    }
}
